using System;


using Syncfusion.Windows.Forms.Grid;


namespace GridSample
{
	/// <summary>
	/// Summary description for FNumericUpDownCellModel.
	/// </summary>
	public class FNumericUpDownCellModel : GridNumericUpDownCellModel
	{
		 

		public FNumericUpDownCellModel(GridModel grid)
			: base(grid)
		{
			 
		}

		public FNumericUpDownCellModel(GridModel grid, int exp)
			: base(grid)
		{
			 
		}
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new FNumericUpDownCellRenderer(control, this);
		}

	}

	public class FNumericUpDownCellRenderer : GridNumericUpDownCellRenderer
	{
		
		private string _format;
		
		bool wrap;
		double minimum, maximum;
		bool hasMinimum, hasMaximum;
		bool hasInitialValue = true;
		double initialValue;
		double step = 1;
		
		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			base.OnInitialize(rowIndex, colIndex);

			
			FNumericUpDownCellInfo ua =  Grid.Model[rowIndex, colIndex].Tag as FNumericUpDownCellInfo ;

			if(ua!= null)
			{
				_format = "{0:F" + ua.DecPlaces.ToString() + "}";
				
				this.wrap = ua.WrapValueF;
				this.minimum = ua.MinimumF;
				this.hasMinimum = true;
				this.maximum = ua.MaximumF; 
				this.hasMaximum = true;
				this.initialValue = ua.StartValueF;
				this.step = ua.StepF;
			}
		}

		public FNumericUpDownCellRenderer(GridControlBase grid, GridCellModelBase cellModel )
			: base(grid, cellModel)
			{
				 
			}

		/// <override/>
		protected override void OnButtonClicked(int rowIndex, int colIndex, int button)
		{
			CurrentCell.BeginEdit();

			string strText = TextBoxText;

			if (!IsReadOnly())
			{
				double lValue = this.minimum;

				try
				{
					lValue = double.Parse(strText);
					if (button == 0)
					{
						if (!this.hasMaximum || lValue <= this.maximum - this.step)
							lValue += this.step;
						else if (this.wrap && this.hasMinimum)
							lValue = this.minimum;
					}
					else
					{
						if (!this.hasMinimum || lValue >= this.minimum + this.step)
							lValue -= this.step;
						else if (this.wrap && this.hasMaximum)
							lValue = this.maximum;
					}
				}
				catch 
				{
					//TraceUtil.TraceExceptionCatched(ex);
					if (this.hasInitialValue)
						lValue = this.initialValue;
					else
						lValue = this.minimum;
				}

				if (this.hasMinimum)
					lValue = Math.Max(this.minimum, lValue);

				if (this.hasMaximum)
					lValue = Math.Min(this.maximum, lValue);


				TextBoxText = string.Format(_format, lValue);
				
			}

			//base.OnButtonClicked(rowIndex, colIndex, button);
		}

		protected override bool OnSaveChanges()
		{
			string strText = TextBoxText;
			bool ok = true;

			double lValue = this.minimum;

			try
			{
				lValue = double.Parse(strText);
				if(lValue < this.minimum || lValue > this.maximum)
					ok = false;
			}
			catch
			{
				ok = false;
			}

			if(ok)
				ok = base.OnSaveChanges();

			return ok;
		}

		protected override void OnKeyPress(System.Windows.Forms.KeyPressEventArgs e)
		{
			base.OnKeyPress(e);

			//digits only in column 2
			if(!char.IsDigit(e.KeyChar))
				e.Handled = true;
		}

	
	}

	public class FNumericUpDownCellInfo : GridNumericUpDownCellInfo
	{
		public int DecPlaces;
		public bool WrapValueF;
		public double MaximumF;
		public double MinimumF;
		public double StartValueF;
		public double StepF;
		 

		public FNumericUpDownCellInfo(double min, double max, double start, double step, bool wrap, int shift1)
		 	//: base(min, max, start, step, wrap)
		{
			WrapValueF = wrap;
			MaximumF = max;
			MinimumF = min;
			StartValueF = start;
			StepF = step;
			DecPlaces = shift1;
		}
	}
}
